from setuptools import setup, find_packages

setup(
    name="coocan",
    version="0.3.3",
    author="wauo",
    author_email="markadc@126.com",
    description="Air Spider Framework",
    packages=find_packages(),
    python_requires=">=3.10",
)
